/******************************************************************************
 * C-BUNG 04
 * Arithmetik, Ein- und Ausgabe an der Konsole
 *           
 * (c) 2003, by Jan Iversen und Andreas Schibilla
 *****************************************************************************/
#include "sometools.h"
#include "ueb04.h"



/******************************************************************************
 * Gibt die Hilfe aus
 *           
 * Die Fkt. gibt den Fehlercode 1 zurck
 *****************************************************************************/
int ShowUsage(int auswahl)
{
 switch(auswahl)
 {
   case 1:  printf(" -f Dichte Temperatur spez. Waermekapazitt [Volumen]\n");
            break;
   case 2:  printf(" -b Behaeltergroesse\n");
            break;
   default: printf("Usage: ueb04 <Opt> [Parameterliste] [ <Opt> [Parameterliste] ] ...\n");
            printf("         -h bzw. -H: Hilfe. Erweiterungsparameter b/B und f/F.\n");
            printf("         -b bzw. -B: Angabe der Behaeltergroesse.\n");
            printf("         -f bzw. -F: Angabe der Fluessigkeitsangaben (Dichte, Temperatur,\n");
            printf("                     spez. Waermekapazitaet und Volumen) mit Mehrfachnennungen.\n");
 }

 return 1;
}



/******************************************************************************
 * Durchluft alle Parameter und ermittelt das Behlter Volumen
 * Bricht bei doppelter Angabe oder ungltiger Syntax das Prog. ab!
 * 
 * [-->] argc           - Anzahl Parameter
 * [-->] argv           - Array mit den Parameter-Strings
 * [<--] wert           - bekommt das Volumen zugewiesen
 * [<->] status         - wird gesetzt, wenn gltiges Volumen gefunden
 *
 * 2003-05-07 ji/as
 *****************************************************************************/
void BehaelterErmitteln(int argc, char * argv[], double * wert, int * status)
{
 int i;

 /* durchlaufe ab 1, da das nullte Element uninteressant ist */
 for (i=1; i<argc; ++i)
   {
       
      /* ist der Parameter ein gltiger Behlter-Parameter ? */
      if( strcicmp(argv[i], "-b") )
         {
            if (*status)                              /* Paramter -b ist doppelt! */
               exit(ShowUsage(0));  

            /* folgt ein Volumenangabe-Param ? */
            /* kann Volumenangabe in Zahl konvertiert werden ? */
            if ( ((argc-i-1) < 1) || (sscanf(argv[i+1], "%lf", wert) != 1) )
               exit(ShowUsage(2));


            if (*wert <= 0.0)                         /* ist die Zahl frs Volumen positiv ? */
               exit(ShowUsage(2));
            
            *status = 1;
         }
   }

}



/******************************************************************************
 * Prft ob das Volumen der Flssigkeit weggelassen werden durfte
 * (also ein -f noch folgt)
 * Rckgabe 1 = okay          
 * Rckgabe 0 = Fehler!
 *****************************************************************************/
int WeglassenOkay(char * argv[], int count)
{
 int i;
 for (i=0; i<count; ++i)
   {
      if (strcicmp(argv[i], "-f") )
         return 0;
   }
 return 1;
}




/******************************************************************************
 * Guckt ob der aktuelle Parameter eine Flssigkeit ist und ob dieser
 * gltig ist. Bei Fehler wird das Prog. abgebrochen
 * 
 * [-->] argc           - Anzahl Parameter
 * [-->] argv           - Array mit den Parameter-Strings
 * [<->] i              - Nr des aktuellen Parameters, nach Bearbeitung eines
 *                        Params wird der wird erhht
 * [<->] groessen       - Enthlt die physikalischen Gren
 * [<->] status         - wird gesetzt, wenn gltige Flssigk. gefunden
 *
 * 2003-05-07 ji/as
 *****************************************************************************/
int IstFluessigkeit(int argc, char * argv[], int * i, double groessen[], int *status)
{
 double  d, T, C, V, m;
 
 /* ist der Parameter ein gltiger Flssigkeits-Parameter ? */
 if( strcicmp(argv[*i],"-f") )
 {
      
      if ((argc-*i-1) < 3)                            /* folgen drei gltige Argumente ? */
            exit(ShowUsage(1));


      /* kann Dichte in Zahl konvertiert werden? */
      if ( (sscanf(argv[*i+1], "%lf", &d) != 1) || (d <= 0.0) )     
            exit(ShowUsage(1));


      /* kann Temp. in Zahl konvertiert werden? */
      if ( (sscanf(argv[*i+2], "%lf", &T) != 1) || (T < -273.0) )     
            exit(ShowUsage(1));

      /* kann spez. Wrmekap. in Zahl konvertiert werden? */
      if ( (sscanf(argv[*i+3], "%lf", &C) != 1) || (C <= 0.0) )     
            exit(ShowUsage(1));

      /* ist kein Volumen als vierter Param angegeben ? */
      if ( ((argc-*i-1) < 4)  || (sscanf(argv[*i+4], "%lf", &V) != 1) ) 
         {
            // prfe, ob das Weglassen okay und ermittle wert fr volumen
            if (WeglassenOkay(&argv[*i+4], argc-*i-4)    &&          /* weglassen ist okay und */
                (groessen[GESAMTVOLUMEN] < groessen[BEHAELTER]) )    /* im Behlter ist noch Platz */
               V = groessen[BEHAELTER] - groessen[GESAMTVOLUMEN];    /* -> dann auffllen */
            else
               exit(ShowUsage(1));  

            (*i)--;                                   /* der letzte Param fehlt! */
         }


      if ( (groessen[BEHAELTER]-groessen[GESAMTVOLUMEN]) > 0.0)      /* wenn noch platz ist */
         if ( (V+groessen[GESAMTVOLUMEN]) > groessen[BEHAELTER])     /* und der behlter berlaufen wrde */
            V = groessen[BEHAELTER] - groessen[GESAMTVOLUMEN];       /* passe Volumen an */

      if (V <= 0)                                     /* Volumen bersteigt Behlter */
          exit(ShowUsage(0));  
      
      
      m = d * V;                                      /* Masse = Dichte mal Volumen */
      groessen[GESAMTMASSE]   += m;
      groessen[GESAMTVOLUMEN] += V;
      groessen[WAERMEINHALT]  += C * m * T;
      groessen[WAERMEANTEIL]  += C * m;

      if (groessen[GESAMTVOLUMEN] > groessen[BEHAELTER])
         exit(ShowUsage(0));  

      *i += 5;                                        /* Vier Kommandoparameter wurden bearbeitet */
      return *status = 1;

 }
 else
      return 0;

}


/******************************************************************************
 * Gibt die entsprechende Hilfe aus und beendet das Programm
 *           
 * Die Fkt. gibt den Fehlercode 1 zurck
 *****************************************************************************/
void HilfeAusgeben(char * param)
{

 if( strcicmp(param,"-hf") )                   /* Hilfe Flssigkeitsangabe */
    exit( !ShowUsage(1) );
      
 if( strcicmp(param,"-hb") )                   /* Hilfe Behltergre */
    exit( !ShowUsage(2) );
      
 if( strcicmp(param,"-h") )                    /* allgemeine Hilfe */
    exit( !ShowUsage(0) );
}



/******************************************************************************
 * Hauptfkt. Prft und berechnet die physik. Gren
 *
 * -> argc     - Wieviele Paramter
 * -> argv     - Array mit den Kommandozeilenparamtern als Strings
 *   
 * Die Fkt. gibt Returncode an Betriebssystem zurck
 *****************************************************************************/
int main (int argc, char * argv[])
{
 int     i=1;
 int     behaelter_status           = 0;
 int     fluessigkeit_status        = 0;
 double  groessen[ANZAHLGROESSEN]   = { 0.0 };
    
 /* keine Parameter an Programm */ 
 if (argc<=1)
     return ShowUsage(0),0;

 
 /* ein Parameter: */
 if (argc==2)
   HilfeAusgeben(argv[i]);


 BehaelterErmitteln(argc, argv, &groessen[BEHAELTER], &behaelter_status);

 /* Parameter parsen */
 while (i<argc)
 {

   if ( strcicmp(argv[i],"-b" ) )                     /* Behlter wurde bereits ermittelt, berspringen */
      {
       i+=2;
       continue;
      }

   /* ist Parameter eine Flssigkeit */
   if ( IstFluessigkeit(argc, argv, &i, groessen, &fluessigkeit_status) )
      continue;

   
   return ShowUsage(0);                               /* Parameter wurde nicht behandelt -> ungltig */
 }


 
 if (!behaelter_status || !fluessigkeit_status)       /* es wurde kein -b oder -f angegeben -> Hilfe */
   return ShowUsage(0);


 printf("Temperatur: %.1f\n",    groessen[WAERMEINHALT] / groessen[WAERMEANTEIL]);
 printf("Gesamtmasse: %.2f\n",   groessen[GESAMTMASSE]);
 printf("Gesamtdichte: %.2f\n",  groessen[GESAMTMASSE] / groessen[GESAMTVOLUMEN]);

/* printf("GESAMTVOLUMEN: %.2f\n",  groessen[GESAMTVOLUMEN]);*/
 

 return 0;
}



